#!/bin/ksh
###################################################################################
# Purpose     : To backup an Oracle Database using RMAN                           #
# Author      : James Moore                                                       #
# Description : Performs an RMAN backup of any specified database and writes the  #
#               output to a log file and then emails the result.                  #
# Usage       : Set the Oracle home variable within the script to the correct     #
#               path.                                                             #
#               Run on the command line or within a crontab as same user that the #
#               database runs as.                                                 #
#               Specify the database SID for the first command line argument and  #
#               the type of backup as the second argument.                        #
#               The type of backup can either be "FULL" which is a whole database #
#               backup, or "INC0" which is an incremental level 0 backup, or      #
#               "INC1" which is an incremental level 1 backup (differential not   #
#               cumulative).                                                      #
#                                                                                 #
#        Syntax: ./rman_backup.sh <SID> <FULL|INC0|INC1>                          #
#                                                                                 #
#        Example: ./rman_backup.sh gistest3 INC0                                  #
#                                                                                 #
###################################################################################

# User-defined variables. Please ensure these are correct before you execute the
# script.

# email address to where the report gets sent
SUPPORT_EMAIL_ADDRESS=support@acme.com

# set to Y if you want to use the recovery catalog
USE_RECOVERY_CATALOG=Y

# the following need to be set if USE_RECOVERY_CATALOG is set to Y
RECOVERY_CATALOG_USR=myrmanusername
RECOVERY_CATALOG_PWD=thepassword
RECOVERY_CATALOG_HOST=myserverhostname
RECOVERY_CATALOG_PORT=1521
RECOVERY_CATALOG_SID=mydbsid

## start of main script

# check there were valid parameters passed in from the command line
if [[ $# != 2 ]]; then
   echo "2 parameters not specified."
   echo "Usage:"
   echo "  $0 <ORACLESID> <FULL0|INC0|INC1>"
   exit 1
elif [[ $2 != "FULL" && $2 != "INC0" && $2 != "INC1" ]]; then
   echo "Invalid backup type specified: 2nd parameter must be string FULL0 or INC0 or INC1"
else
   ORACLE_SID=$1
fi

# Try to automatically find ORACLE_HOME. If this fails you can
# just hard code it in.
ORACLE_HOME=`grep $ORACLE_SID /etc/oratab | cut -f2 -d':' | tail -1`

# export necessary environment variables
export ORACLE_SID
export ORACLE_HOME
LD_LIBRARY_PATH="$ORACLE_HOME/lib32:$ORACLE_HOME/lib"; export LD_LIBRARY_PATH
PATH="$ORACLE_HOME/bin:$ORACLE_HOME/lib32:/usr/bin:/etc:/usr/sbin:/usr/ucb:$HOME/bin:/usr/bin/X11:/sbin"; export PATH

# construct the recovery catalog connection string
RECOVERY_CATALOG_CONN_STRING=$RECOVERY_CATALOG_USR/$RECOVERY_CATALOG_PWD@(description=(address=(host=$RECOVERY_CATALOG_HOST)(protocol=tcp)(port=$RECOVERY_CATALOG_PORT))(connect_data=(sid=$RECOVERY_CATALOG_SID)))
if [[ $USE_RECOVERY_CATALOG == "Y" ]]; then
   RCATCMDSTR="connect catalog $RECOVERY_CATALOG_CONN_STRING"
else
   RCATCMDSTR=""
fi

# log to the directory the script is in.
LOGDIR=`dirname $0`
LOGFILE="${LOGDIR}/${ORACLE_SID}_rman_backup.log"

# if its a full backup then
if [[ $2 == "FULL" ]]; then
   echo "Starting backup of whole database and archive logs at `date`" > $LOGFILE
   echo "Current directory is $PWD" >> $LOGFILE
   echo "Oracle SID: ${ORACLE_SID}" >> $LOGFILE
   echo "Oracle Home: ${ORACLE_HOME}" >> $LOGFILE
   echo "\n" >> $LOGFILE
   rman target=/ > $LOGFILE << EOF
$RCATCMDSTR
SQL "ALTER SYSTEM ARCHIVE LOG CURRENT";
BACKUP CHECK LOGICAL DATABASE PLUS ARCHIVELOG DELETE ALL INPUT;
DELETE NOPROMPT OBSOLETE;
EXIT
EOF
# else if its an incremental level 0 backup then
elif [[ $2 == "INC0" ]]; then
   echo "Starting level 0 incremental backup of database and archive logs at `date`" > $LOGFILE
   echo "Current directory is $PWD" >> $LOGFILE
   echo "Oracle SID: ${ORACLE_SID}" >> $LOGFILE
   echo "Oracle Home: ${ORACLE_HOME}" >> $LOGFILE
   echo "\n" >> $LOGFILE
   rman target=/ >> $LOGFILE << EOF
$RCATCMDSTR
SQL "ALTER SYSTEM ARCHIVE LOG CURRENT";
BACKUP CHECK LOGICAL INCREMENTAL LEVEL 0 DATABASE PLUS ARCHIVELOG DELETE ALL INPUT;
DELETE NOPROMPT OBSOLETE;
EXIT
EOF
# else it must be an incremental level 1 backup
else
   echo "Starting level 1 incremental backup of database and archive logs at `date`" > $LOGFILE
   echo "Current directory is $PWD" >> $LOGFILE
   echo "Oracle SID: ${ORACLE_SID}" >> $LOGFILE
   echo "Oracle Home: ${ORACLE_HOME}" >> $LOGFILE
   echo "\n" >> $LOGFILE
   rman target=/ >> $LOGFILE << EOF
$RCATCMDSTR
SQL "ALTER SYSTEM ARCHIVE LOG CURRENT";
BACKUP CHECK LOGICAL INCREMENTAL LEVEL 1 DATABASE PLUS ARCHIVELOG DELETE ALL INPUT;
DELETE NOPROMPT OBSOLETE;
EXIT
EOF
fi
echo "Backup finished at `date`" >> $LOGFILE

# check for errors
if [ `grep -c RMAN-00569 $LOGFILE` -gt 0 ]; then
   SUBJECTHEADER="RMAN BACKUP : ATTENTION!: The $2 rman backup of the $ORACLE_SID database has failed"
else
   if [ `grep -ic error $LOGFILE` -gt 0 ]; then
      SUBJECTHEADER="RMAN BACKUP : ATTENTION!: The $2 rman backup of the $ORACLE_SID database has failed"
   else
      SUBJECTHEADER="RMAN BACKUP : The $2 rman backup of the $ORACLE_SID database was OK"
   fi
fi

# e-mail the result to apps support
cat $LOGFILE | /usr/bin/mailx -s "$SUBJECTHEADER" $SUPPORT_EMAIL_ADDRESS
